@@ -18,9 +18,8 @@
 #' @importFrom aod wald.test
 #' @importFrom car Anova
 #' @importFrom lmtest waldtest
-#' @importFrom regtools nlshc
 #' @importFrom sandwich vcovHC
-#' @importFrom stats anova as.formula coef formula glm hatvalues nobs pchisq pf predict quantile rbinom residuals rnorm runif update vcov
+#' @importFrom stats anova as.formula coef formula glm hatvalues lm nobs pchisq pf predict quantile rbinom residuals rnorm runif update vcov
 #' @importFrom utils capture.output
 #' @export
 #' @details The RESI, denoted as S, is applicable across many model types. It is a unitless
@@ -47,6 +46,11 @@
 #' \code{\link{t2S}}, \code{\link{z2S}}, \code{\link{t2S_alt}}, and
 #' \code{\link{z2S_alt}} for more details on the formulas.
 #'
+#' For GEE (\code{geeglm}) models, a longitudinal RESI (L-RESI) and a cross-sectional,
+#' per-measurement RESI (CS-RESI) is estimated. The longitudinal RESI takes the
+#' specified clustering into account, while the cross-sectional RESI is estimated
+#' using a model where each measurement is its own cluster.
+#'
 #' For most \code{lm} and \code{nls} model types, there is a Bayesian bootstrap
 #' option available as an alternative to the default, standard non-parametric
 #' bootstrap. The interpretation of a Bayesian bootstrapped interval is similar to
@@ -63,7 +67,7 @@
 #' @return Returns a list that includes function arguments, RESI point estimates,
 #' and confidence intervals in coefficients/anova-style tables
 #' @family RESI functions
-#' @seealso \code{\link{resi_pe}}, \code{\link{vcovHC}}, \code{\link{nlshc}},
+#' @seealso \code{\link{resi_pe}}, \code{\link{vcovHC}},
 #' \code{\link{f2S}}, \code{\link{chisq2S}}, \code{\link{z2S}}, \code{\link{t2S}}
 #'
 #' @examples
@@ -137,7 +141,6 @@ resi.default <- function(model.full, model.reduced = NULL, data, anova = TRUE,
                          coefficients = TRUE, nboot = 1000,
                          vcovfunc = sandwich::vcovHC, alpha = 0.05, store.boot = FALSE,
                          Anova.args = list(), vcov.args = list(), unbiased = TRUE, ...){
-
   dots = list(...)
   if ("boot.method" %in% names(dots)){
     message("Only nonparametric bootstrap supported for model type")
@@ -192,7 +195,6 @@ resi.default <- function(model.full, model.reduced = NULL, data, anova = TRUE,
     CIs = apply(boot.results[,(ncol(boot.results)-length(which(rownames(output$anova) != "Residuals"))+1):ncol(boot.results)], 2,  quantile, probs = alpha.order, na.rm = TRUE)
     CIs = t(CIs)
     output$anova[1:nrow(CIs), paste(alpha.order*100, '%', sep='')] = CIs
-    class(output$anova) = c("anova_resi", class(output$anova))
   }
 
   if(store.boot){
@@ -281,7 +283,6 @@ resi.lm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
     CIs = apply(boot.results[,(ncol(boot.results)-length(which(rownames(output$anova) != "Residuals"))+1):ncol(boot.results)], 2,  quantile, probs = alpha.order, na.rm = TRUE)
     CIs = t(CIs)
     output$anova[1:nrow(CIs), paste(alpha.order*100, '%', sep='')] = CIs
-    class(output$anova) = c("anova_resi", class(output$anova))
   }
 
   if(store.boot){
@@ -295,7 +296,7 @@ resi.lm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
 #' @export
 resi.nls <- function(model.full, model.reduced = NULL, data, coefficients = TRUE,
                      nboot = 1000, boot.method = 'nonparam',
-                     vcovfunc = regtools::nlshc, alpha = 0.05, store.boot = FALSE,
+                     vcovfunc = vcovnls, alpha = 0.05, store.boot = FALSE,
                      vcov.args = list(), unbiased = TRUE, ...){
 
   boot.method = match.arg(tolower(boot.method), choices = c("nonparam", "bayes"))
@@ -314,15 +315,36 @@ resi.nls <- function(model.full, model.reduced = NULL, data, coefficients = TRUE
   # bootstrapping
   boot.results = data.frame(matrix(nrow = nboot, ncol = length(output$estimates)))
   colnames(boot.results) = names(output$estimates)
+  fail = 0
   # non-parametric bootstrap
   if (tolower(boot.method) == "nonparam"){
     for (i in 1:nboot){
+
       boot.data = boot.samp(data)
-      boot.model.full <- update(model.full, data = boot.data)
-      boot.results[i,] = suppressWarnings(resi_pe(model.full = boot.model.full, model.reduced = NULL,
-                                                  data = boot.data, coefficients = coefficients,
-                                                  vcovfunc = vcovfunc, vcov.args = vcov.args,
-                                                  unbiased = unbiased, ...)$estimates)
+      t <- try(update(model.full, data = boot.data), silent = T)
+      if (inherits(t, "try-error")){
+        fail = fail + 1
+        boot.results[i,] = NA
+      }
+      else{
+        boot.model.full <- update(model.full, data = boot.data)
+
+        # for catching overall error
+        t1 <- try(suppressWarnings(resi_pe(model.full = boot.model.full, model.reduced = NULL,
+                          data = boot.data, coefficients = coefficients,
+                          vcovfunc = vcovfunc, vcov.args = vcov.args,
+                          unbiased = unbiased, ...)$estimates), silent = T)
+        if (inherits(t1, "try-error")){
+          fail = fail + 1
+        }
+        else{
+          boot.results[i,] = suppressWarnings(resi_pe(model.full = boot.model.full, model.reduced = NULL,
+                                                      data = boot.data, coefficients = coefficients,
+                                                      vcovfunc = vcovfunc, vcov.args = vcov.args,
+                                                      unbiased = unbiased, ...)$estimates)
+        }
+
+      }
 
     }}
 
@@ -331,13 +353,29 @@ resi.nls <- function(model.full, model.reduced = NULL, data, coefficients = TRUE
     g = NULL
     for (i in 1:nboot){
       boot.data = bayes.samp(data)
-      boot.model.full <- update(model.full, data = boot.data, weights = g)
-      boot.results[i,] = suppressWarnings(resi_pe(model.full = boot.model.full, model.reduced = NULL,
-                                                  data = boot.data, coefficients = coefficients,
-                                                  vcovfunc = vcovfunc, vcov.args = vcov.args,
-                                                  unbiased = unbiased, ...)$estimates)
 
-    }}
+      t <- try(update(model.full, data = boot.data, weights = g), silent = T)
+      if (inherits(t, "try-error")){
+        fail = fail + 1
+        boot.results[i,] = NA
+      }
+      else{
+        boot.model.full <- update(model.full, data = boot.data, weights = g)
+
+        # for catching overall error
+        t1 <- try(resi_pe(model.full = boot.model.full, model.reduced = NULL,
+                          data = boot.data, coefficients = coefficients,
+                          vcovfunc = vcovfunc, vcov.args = vcov.args,
+                          unbiased = unbiased, ...)$estimates, silent = T)
+        if (inherits(t1, "try-error")){
+          fail = fail + 1
+        }
+        else{
+          boot.results[i,] = suppressWarnings(resi_pe(model.full = boot.model.full, model.reduced = NULL,
+                                                      data = boot.data, coefficients = coefficients,
+                                                      vcovfunc = vcovfunc, vcov.args = vcov.args,
+                                                      unbiased = unbiased, ...)$estimates)
+        }}}}
 
   alpha.order = sort(c(alpha/2, 1-alpha/2))
   output$overall[nrow(output$overall),paste(alpha.order*100, '%', sep='')] = quantile(boot.results[,1], probs = alpha.order, na.rm = TRUE)
@@ -351,6 +389,7 @@ resi.nls <- function(model.full, model.reduced = NULL, data, coefficients = TRUE
   if(store.boot){
     output$boot.results = boot.results
   }
+  output$nfail = fail
   class(output) = "resi"
   return(output)
 }
@@ -380,7 +419,7 @@ resi.survreg <- function(model.full, model.reduced = NULL, data, anova = TRUE,
   resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
                anova = anova, coefficients = coefficients, nboot = nboot, vcovfunc = vcovfunc,
                store.boot = store.boot, Anova.args = Anova.args,
-               unbiased = unbiased, ...)
+               unbiased = unbiased, alpha = alpha, ...)
 }
 
 #' @describeIn resi RESI point and interval estimation for coxph models
@@ -407,7 +446,7 @@ resi.coxph <- function(model.full, model.reduced = NULL, data, anova = TRUE,
   resi.default(model.full = model.full, model.reduced = model.reduced, data = data,
                anova = anova, coefficients = coefficients, nboot = nboot, vcovfunc = vcovfunc,
                store.boot = store.boot, Anova.args = Anova.args,
-               unbiased = unbiased, ...)
+               unbiased = unbiased, alpha = alpha, ...)
 }
 
 #' @describeIn resi RESI point and interval estimation for hurdle models
@@ -477,57 +516,146 @@ resi.zeroinfl <- resi.hurdle
 
 #' @describeIn resi RESI point and interval estimation for GEE models
 #' @export
-resi.geeglm <- function(model.full, alpha = 0.05, nboot = 1000, ...){
-  warning("\nInterval performance not yet evaluated for geeglm")
-  output <- list(alpha = alpha, nboot = nboot)
-  # RESI point estimates
-  output = c(output, resi_pe(model.full))
-  data = model.full$data
+
+resi.geeglm <- function(model.full, data, anova = TRUE,
+                        coefficients = TRUE, nboot = 1000,
+                        alpha = 0.05, store.boot = FALSE,
+                        unbiased = TRUE, ...){
+  dots = list(...)
+  if ("boot.method" %in% names(dots)){
+    message("Only nonparametric bootstrap supported for model type")
+  }
+
+  if (missing(data)){
+    data = model.full$data
+    tryCatch(update(model.full, data = data), error = function(e){
+      message("Updating model fit failed. Try rerunning with providing data argument")})
+  }
+  else{
+    data = as.data.frame(data)
+  }
+
+  # point estimation
+  output <- list(alpha = alpha, nboot = nboot, boot.method = "nonparam")
+  output = c(output, resi_pe(model.full = model.full, data = data, anova = anova,
+                             coefficients = coefficients, unbiased = unbiased, ...))
+
   # id variable name
   id_var = as.character(model.full$call$id)
-  # bootstrap
-  output.boot = as.matrix(output$coefficients[, 'RESI'])
+  corstr_spec = model.full$corstr
+  # bootstrapping
+  boot.results = data.frame(matrix(nrow = nboot, ncol = length(output$estimates)))
+  colnames(boot.results) = names(output$estimates)
+  fail = 0
   for (i in 1:nboot){
+    skip_to_next <- FALSE
     boot.data = boot.samp(data, id.var = id_var)
     # re-fit the model
-    boot.mod = update(model.full, data = boot.data)
-    output.boot = cbind(output.boot, resi_pe(boot.mod)$coefficients[, 'RESI'])
+    boot.model.full = update(model.full, data = boot.data, corstr = corstr_spec,
+                             id = boot.data$bootid)
+    rv.boot = tryCatch(resi_pe(boot.model.full,  data = boot.data, anova = anova,
+                               coefficients = coefficients, unbiased = unbiased, ...),
+                       error = function(e){skip_to_next <<- TRUE})
+    if (skip_to_next) {
+      fail = fail + 1
+      next}
+    #output.boot$RESI= cbind(output.boot$RESI, rv.boot$resi[, 'RESI'])
+    boot.results[i,] = suppressWarnings(resi_pe(model.full = boot.model.full,
+                                                data = boot.data, anova = anova,
+                                                coefficients = coefficients,
+                                                unbiased = unbiased, ...)$estimates)
   }
-  output.boot = output.boot[, -1]
-  RESI.ci = apply(output.boot, 1, quantile, probs = c(alpha/2, 1-alpha/2), na.rm = TRUE)
-  output$coefficients = cbind(output$coefficients, t(RESI.ci))
-  output$boot.method = "nonparam"
-  class(output)= 'resi'
+
+  alpha.order = sort(c(alpha/2, 1-alpha/2))
+
+  if (coefficients){
+    lCIs = apply(boot.results[,1:nrow(output$coefficients)], 2,  quantile,
+                 probs = alpha.order, na.rm = TRUE)
+    lCIs = t(lCIs)
+    output$coefficients[1:nrow(lCIs), paste("L ",alpha.order*100, '%', sep='')] = lCIs
+    cCIs = apply(boot.results[,(nrow(output$coefficients)+1):(2*nrow(output$coefficients))],
+                 2,  quantile, probs = alpha.order, na.rm = TRUE)
+    cCIs = t(cCIs)
+    output$coefficients[1:nrow(cCIs), paste("CS ",alpha.order*100, '%', sep='')] = cCIs
+  }
+
+  if (anova){
+    lCIs = apply(boot.results[,(ncol(boot.results)-
+                                  2*length(rownames(output$anova))+1):
+                                (ncol(boot.results)-length(rownames(output$anova)))],
+                 2,  quantile, probs = alpha.order, na.rm = TRUE)
+    lCIs = t(lCIs)
+    output$anova[1:nrow(lCIs), paste("L ", alpha.order*100, '%', sep='')] = lCIs
+    cCIs = apply(boot.results[,(ncol(boot.results)-length(rownames(output$anova))+1):
+                                ncol(boot.results)],
+                 2,  quantile, probs = alpha.order, na.rm = TRUE)
+    cCIs = t(cCIs)
+    output$anova[1:nrow(cCIs), paste("CS ", alpha.order*100, '%', sep='')] = cCIs
+  }
+
+  if(store.boot){
+    output$boot.results = boot.results
+  }
+
+  output$nfail = fail
+
+  class(output) = "resi"
+  return(output)
+
+  # RESI_se = apply(output.boot$RESI, 1, sd, na.rm = TRUE)
+
   return(output)
 }
 
+
 #' @describeIn resi RESI point and interval estimation for GEE models
 #' @export
-resi.gee <- function(model.full, data, alpha = 0.05, nboot = 1000, ...){
+resi.gee <- function(model.full, data, nboot = 1000, alpha = 0.05,
+                     store.boot = FALSE, unbiased = TRUE, ...){
   if (missing(data)){
     stop('\nData argument is required for GEE models from gee package')
   }
   else{
     data = as.data.frame(data)
   }
-  warning("\nInterval performance not yet evaluated for gee")
-  output <- list(alpha = alpha, nboot = nboot)
+
+  output <- list(alpha = alpha, nboot = nboot, boot.method = "nonparam")
   # RESI point estimates
-  output = c(output, resi_pe(model.full))
+  output = c(output, resi_pe(model.full, data, unbiased, ...))
   # id variable name
   id_var = as.character(model.full$call$id)
-  # bootstrap
-  output.boot = as.matrix(output$coefficients[, 'RESI'])
+
+  # bootstrapping
+  boot.results = data.frame(matrix(nrow = nboot, ncol = length(output$estimates)))
+  colnames(boot.results) = names(output$estimates)
   for (i in 1:nboot){
     boot.data = boot.samp(data, id.var = id_var)
     # re-fit the model
-    suppressMessages(capture.output(boot.mod <- update(model.full, data = boot.data), file =  nullfile()))
-    output.boot = cbind(output.boot, resi_pe(boot.mod)$coefficients[, 'RESI'])
+    suppressMessages(capture.output(boot.model.full <-
+                                      update(model.full, data = boot.data,
+                                             id = boot.data$bootid),
+                                    file =  nullfile()))
+    boot.results[i,] = suppressWarnings(resi_pe(model.full = boot.model.full,
+                                                data = boot.data,
+                                                unbiased = unbiased, ...)$estimates)
   }
-  output.boot = output.boot[, -1]
-  RESI.ci = apply(output.boot, 1, quantile, probs = c(alpha/2, 1-alpha/2), na.rm = TRUE)
-  output$coefficients = cbind(output$coefficients, t(RESI.ci))
-  output$boot.method = "nonparam"
+
+  alpha.order = sort(c(alpha/2, 1-alpha/2))
+
+  lCIs = apply(boot.results[,1:nrow(output$coefficients)], 2,  quantile,
+                probs = alpha.order, na.rm = TRUE)
+  lCIs = t(lCIs)
+  output$coefficients[1:nrow(lCIs), paste("L ", alpha.order*100, '%', sep='')] = lCIs
+  cCIs = apply(boot.results[,(nrow(output$coefficients) + 1):
+                              (2 * nrow(output$coefficients))], 2,  quantile,
+               probs = alpha.order, na.rm = TRUE)
+  cCIs = t(cCIs)
+  output$coefficients[1:nrow(cCIs), paste("CS ", alpha.order*100, '%', sep='')] = cCIs
+
+  if(store.boot){
+    output$boot.results = boot.results
+  }
+
   class(output)= 'resi'
   return(output)
 }
@@ -539,54 +667,59 @@ resi.gee <- function(model.full, data, alpha = 0.05, nboot = 1000, ...){
 #' @export
 resi.lme <- function(model.full, alpha = 0.05, nboot = 1000, vcovfunc = clubSandwich::vcovCR,
                      vcov.args = list(), ...){
-  warning("\nInterval performance not yet evaluated for lme")
-  output <- list(alpha = alpha, nboot = nboot)
+  warning("\nConfidence Interval procedure not developed for lme, returning point estimates only")
+  output <- list(alpha = alpha, nboot = 0)
   # RESI point estimates
   output = c(output, resi_pe(model.full = model.full, vcovfunc = vcovfunc, vcov.args = vcov.args))
   data = model.full$data
   # id variable name
   id_var = attr(nlme::getGroups(model.full), "label")
   # bootstrap
-  output.boot = as.matrix(output$coefficients[, 'RESI'])
-  fun <- utils::getFromNamespace("update.lme", "nlme")
-  tryCatch(fun(model.full, data = data), error = function(e){
-    message("Try running `library(nlme)`")})
-  for (i in 1:nboot){
-    boot.data = boot.samp(data, id.var = id_var)
-    # re-fit the model
-    boot.mod = fun(model.full, data = boot.data, fixed = as.formula(model.full$call$fixed),
-                      random = as.formula(model.full$call$random))
-    output.boot = cbind(output.boot, resi_pe(model.full = boot.mod, vcovfunc = vcovfunc,
-                                             vcov.args = vcov.args)$coefficients[, 'RESI'])
-  }
-  output.boot = output.boot[, -1]
-  RESI.ci = apply(output.boot, 1, quantile, probs = c(alpha/2, 1-alpha/2), na.rm = TRUE)
-  output$coefficients = cbind(output$coefficients, t(RESI.ci))
+  #output.boot = as.matrix(output$coefficients[, 'RESI'])
+  #fun <- utils::getFromNamespace("update.lme", "nlme")
+  #tryCatch(fun(model.full, data = data), error = function(e){
+    #message("Try running `library(nlme)`")})
+  # for (i in 1:nboot){
+  #   boot.data = boot.samp(data, id.var = id_var)
+  #   # re-fit the model
+  #   boot.mod = fun(model.full, data = boot.data, fixed = as.formula(model.full$call$fixed),
+  #                     random = as.formula(model.full$call$random))
+  #   output.boot = cbind(output.boot, resi_pe(model.full = boot.mod, vcovfunc = vcovfunc,
+  #                                            vcov.args = vcov.args)$coefficients[, 'RESI'])
+  # }
+  # output.boot = output.boot[, -1]
+  # RESI.ci = apply(output.boot, 1, quantile, probs = c(alpha/2, 1-alpha/2), na.rm = TRUE)
+  # output$coefficients = cbind(output$coefficients, t(RESI.ci))
   output$boot.method = "nonparam"
   class(output) = 'resi'
   return(output)
 }
 
+#' @describeIn resi RESI point and interval estimation for lmerMod models
 #' @export
 resi.lmerMod <- function(model.full, alpha = 0.05, nboot = 1000,
                          vcovfunc = clubSandwich::vcovCR, vcov.args = list(), ...){
-  warning("\nInterval performance not yet evaluated for lmerMod")
-  output = resi_pe(model.full, vcovfunc = vcovfunc, vcov.args = vcov.args) # RESI point estimates
+  warning("\nConfidence Interval procedure not developed for lmerMod, returning point estimates only")
+  output <- list(alpha = alpha, nboot = 0)
+  output = c(output, resi_pe(model.full, vcovfunc = vcovfunc, vcov.args = vcov.args)) # RESI point estimates
   data = model.full@frame
   # id variable name
   id_var = names(model.full@flist)
-  # bootstrap (non-param for now)
-  output.boot = as.matrix(output$coefficients[, 'RESI'])
-  for (i in 1:nboot){
-    boot.data = boot.samp(data, id.var = id_var)
-    # re-fit the model
-    boot.mod = update(model.full, data = boot.data)
-    rv.boot = resi_pe(boot.mod, vcovfunc = vcovfunc, vcov.args = vcov.args)
-    output.boot = cbind(output.boot, rv.boot$coefficients[, 'RESI'])
-  }
-  output.boot = output.boot[, -1]
-  RESI.ci = apply(output.boot, 1, quantile, probs = c(alpha/2, 1-alpha/2))
-  output = list(coefficients = cbind(output$coefficients, t(RESI.ci)), naive.var = output$naive.var)
 
+  # bootstrap (non-param for now)
+  # output.boot = as.matrix(output$coefficients[, 'RESI'])
+  # for (i in 1:nboot){
+  #   boot.data = boot.samp(data, id.var = id_var)
+  #   # re-fit the model
+  #   boot.mod = update(model.full, data = boot.data)
+  #   rv.boot = resi_pe(boot.mod, vcovfunc = vcovfunc, vcov.args = vcov.args)
+  #   output.boot = cbind(output.boot, rv.boot$coefficients[, 'RESI'])
+  # }
+  # output.boot = output.boot[, -1]
+  # RESI.ci = apply(output.boot, 1, quantile, probs = c(alpha/2, 1-alpha/2))
+  # output = c(output, list(coefficients = cbind(output$coefficients, t(RESI.ci)),
+                          # naive.var = output$naive.var))
+  output$boot.method = "nonparam"
+  class(output) = 'resi'
   return(output)
 }
