@@ -17,7 +17,6 @@
 #' @importFrom aod wald.test
 #' @importFrom car Anova
 #' @importFrom lmtest waldtest
-#' @importFrom regtools nlshc
 #' @importFrom sandwich vcovHC
 #' @importFrom stats coef df.residual formula glm hatvalues pf predict quantile residuals update vcov
 #' @export
@@ -47,6 +46,11 @@
 #' \code{\link{t2S}}, \code{\link{z2S}}, \code{\link{t2S_alt}}, and
 #' \code{\link{z2S_alt}} for more details on the formulas.
 #'
+#' For GEE (\code{geeglm}) models, a longitudinal RESI (L-RESI) and a cross-sectional,
+#' per-measurement RESI (CS-RESI) is estimated. The longitudinal RESI takes the
+#' specified clustering into account, while the cross-sectional RESI is estimated
+#' using a model where each measurement is its own cluster.
+#'
 #' @return Returns a list containing RESI point estimates
 #' @examples
 #' # This function produces point estimates for the RESI. The resi function will
@@ -171,6 +175,7 @@ resi_pe.glm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
   else{
     output$naive.var = FALSE
   }
+  class(output) = "resi"
   return(output)
 }
 
@@ -206,11 +211,11 @@ resi_pe.lm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
   }
 
   # overall
-  wald.test = lmtest::waldtest(model.reduced, model.full, vcov = vcovfunc2, test = 'Chisq')
-  stats = wald.test$Chisq[2]
+  wald.test = lmtest::waldtest(model.reduced, model.full, vcov = vcovfunc2, test = 'F')
+  stats = wald.test$F[2]
   overall.df = wald.test$Df[2]
   res.df = wald.test$Res.Df[2]
-  overall.resi.hat = chisq2S(stats, overall.df, nrow(data))
+  overall.resi.hat = f2S(stats, overall.df, res.df, nrow(data))
   wald.test[2, 'RESI'] = overall.resi.hat
   output <- list(model.full = list(call = model.full$call, formula = formula(model.full)),
                  model.reduced = list(call = model.reduced$call, formula = formula(model.reduced)),
@@ -227,7 +232,7 @@ resi_pe.lm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
     if (unbiased){
       coefficients.df[,'RESI'] = t2S(coefficients.df[,'t value'], nrow(data), model.full$df.residual)
     } else{
-      coefficients.df[,'RESI'] = t2S_alt(coefficients.df[,'t value'], model.full$df.residual)
+      coefficients.df[,'RESI'] = t2S_alt(coefficients.df[,'t value'], model.full$df.residual, nrow(data))
     }
 
     output$coefficients = coefficients.df
@@ -240,7 +245,7 @@ resi_pe.lm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
   if (anova){
     suppressMessages(anova.tab <- do.call(car::Anova, c(list(mod = model.full,
                                               vcov. = vcovfunc2), Anova.args)))
-    anova.tab[,'RESI'] = f2S(anova.tab[,'F'], anova.tab[,'Df'], res.df)
+    anova.tab[,'RESI'] = f2S(anova.tab[,'F'], anova.tab[,'Df'], res.df, nrow(data))
     output$anova = anova.tab[which(rownames(anova.tab) != "Residuals"),]
     output$estimates = c(output$estimates, anova.tab$RESI)
     names.est = c(names.est, rownames(anova.tab))
@@ -261,7 +266,7 @@ resi_pe.lm <- function(model.full, model.reduced = NULL, data, anova = TRUE,
 #' @describeIn resi_pe RESI point estimation for nonlinear least squares models
 #' @export
 resi_pe.nls <- function(model.full, model.reduced = NULL, data, coefficients = TRUE,
-                        vcovfunc = regtools::nlshc, vcov.args = list(), unbiased = TRUE, ...){
+                        vcovfunc = vcovnls, vcov.args = list(), unbiased = TRUE, ...){
   if (missing(data) | is.null(data)){
     stop('\nData argument is required for nls model')
   }
@@ -273,7 +278,13 @@ resi_pe.nls <- function(model.full, model.reduced = NULL, data, coefficients = T
   }
 
   if (identical(vcovfunc, sandwich::vcovHC)){
-    vcovfunc = regtools::nlshc
+    # vcovfunc = regtools::nlshc
+    ## current issue with reverse dependencies coming through regtools
+    ## for now copy needed function and move regtools to suggests to
+    ## avoid package potentially getting archived until issue is fixed
+    ## copied code in internal_functions.R
+    vcovfunc = vcovnls
+
     warning("Sandwich vcov function not applicable for nls model type, vcovfunc set to regtools::nlshc")
   }
 
@@ -310,12 +321,12 @@ resi_pe.nls <- function(model.full, model.reduced = NULL, data, coefficients = T
   if (coefficients){
     coefficients.tab <- lmtest::coeftest(model.full, vcov. = vcovmat)
     coefficients.df = data.frame(coefficients.tab[,'Estimate'], coefficients.tab[,'Std. Error'],
-                            coefficients.tab[,'t value'], coefficients.tab[,'Pr(>|t|)'], row.names = rownames(coefficients.tab))
+                                 coefficients.tab[,'t value'], coefficients.tab[,'Pr(>|t|)'], row.names = rownames(coefficients.tab))
     colnames(coefficients.df) = colnames(coefficients.tab)
     if (unbiased){
       coefficients.df[,'RESI'] = t2S(coefficients.df[,'t value'], nrow(data), res.df)
     } else{
-      coefficients.df[,'RESI'] = t2S_alt(coefficients.df[,'t value'], res.df)
+      coefficients.df[,'RESI'] = t2S_alt(coefficients.df[,'t value'], res.df, nrow(data))
     }
     output$coefficients = coefficients.df
     output$estimates = c(output$estimates, coefficients.df$RESI)
@@ -364,30 +375,10 @@ resi_pe.survreg <- function(model.full, model.reduced = NULL, data, anova = TRUE
 
   # Anova table (Chi sq statistics)
   if (anova){
-    # suppressMessages(anova.tab <- car::Anova(model.full, test.statistic = "Wald", error.df = df.residual(model.full), ...))
-    # above currently not working (problem with being able to specify error.df), having to use code from
-    # car:::Anova.II.Wald.survreg
-    V <- vcov(model.full, complete = FALSE)
-    b <- coef(model.full)
-    if (length(b) != nrow(V)) {
-      p <- which(grepl("^Log\\(scale", rownames(V)))
-      if (length(p) > 0)
-        V <- V[-p, -p]
-    }
-
-    if ("type" %in% names(Anova.args)){
-      if (Anova.args$type %in% c(3, "III")){
-        fun <- utils::getFromNamespace("Anova.III.default", "car")
-      }
-      else{
-        fun <- utils::getFromNamespace("Anova.III.default", "car")
-      }
-    } else{
-      fun <- utils::getFromNamespace("Anova.II.default", "car")
-    }
-
-    anova.tab <- do.call(fun, c(list(mod = model.full, vcov. = V, test = "Chisq",
-                                     error.df = df.residual(model.full)), Anova.args))
+    anova.tab <- suppressMessages(do.call(car::Anova, c(list(mod = model.full,
+                                            test.statistic = 'Wald',
+                                            error.df = df.residual(model.full)),
+                                       Anova.args)))
     anova.tab[,'RESI'] = chisq2S(anova.tab[,'Chisq'], anova.tab[,'Df'], nrow(data))
     output$anova = anova.tab[which(rownames(anova.tab) != "Residuals"),]
     names.est = names(output$estimates)
@@ -561,24 +552,161 @@ resi_pe.zeroinfl <- resi_pe.hurdle
 
 #' @describeIn resi_pe RESI point estimation for geeglm object
 #' @export
-resi_pe.geeglm <- function(model.full, ...){
-  x = as.matrix(summary(model.full)$coefficients)
-  #sample size
+resi_pe.geeglm <- function(model.full, data, anova = TRUE,
+                           coefficients = TRUE, unbiased = TRUE, ...){
+  data = model.full$model
+  # sample size
   N = length(summary(model.full)$clusz)
+  # total num of observations
+  tot_obs = nrow(data)
+
   output <- list(model.full = list(call = model.full$call, formula = formula(model.full)),
-                 coefficients =  as.data.frame(cbind(x, RESI = RESI::chisq2S(x[, 'Wald'], 1, N))))
+                 estimates = c())
+  names.est = c()
+
+  # model form
+  form = formula(model.full)
+  # independence model
+  w_resi = model.full$prior.weights
+  data$w_resi = w_resi
+  mod_indg = glm(formula = form, family = model.full$family, data = data,
+                 weights = w_resi, contrasts = model.full$contrasts)
+
+  # the var-cov matrix estimate from the independence model
+  # Note: this is the estimate for Cov[(\hat{\beta}_ind - \beta_0)]
+  cov_ind = sandwich::vcovHC(mod_indg, type = "HC0")
+  cov_ind = cov_ind * tot_obs/N
+  # make copy of model.full, replace vbeta with independence
+  mod_ind = model.full
+  mod_ind$geese$vbeta = cov_ind
+
+  # The var-cov matrix estimate from the analysis model (the one considering correlation )
+  # Note: this is the estimate for Cov(\hat{\beta}_long)
+  cov_long = vcov(model.full)
+  # convert it to the estimate for \Sigma_long = Cov(\sqrt{N}(\hat{\beta}_long - \beta_0))
+  cov_long = cov_long * N
+
+  # longitudinal RESI
+  # coefficients (z statistics)
+  if (coefficients) {
+    # longitudinal resi
+    coefficients.tab <- lmtest::coeftest(model.full)
+    coefficients.df = data.frame(coefficients.tab[,'Estimate'],
+                                 coefficients.tab[,'Std. Error'],
+                                 coefficients.tab[,'z value'],
+                                 coefficients.tab[,'Pr(>|z|)'],
+                                 row.names = rownames(coefficients.tab))
+    colnames(coefficients.df) = colnames(coefficients.tab)
+    if (unbiased){
+      coefficients.df[,'L-RESI'] = z2S(coefficients.df[,'z value'], N)
+    } else{
+      coefficients.df[,'L-RESI'] = suppressWarnings(z2S_alt(coefficients.df[,'z value'],
+                                                            N))
+    }
+
+    # CS RESI
+    # M: use independence mod
+    coefficients.tabcs <- lmtest::coeftest(mod_ind, vcov. = cov_ind)
+    z_cs = coefficients.tabcs[,'z value']
+    if (unbiased){
+      coefficients.df[,'CS-RESI'] = z2S(z_cs, N)
+    } else{
+      coefficients.df[,'CS-RESI'] = suppressWarnings(z2S_alt(z_cs,
+                                                             N))
+    }
+
+    output$coefficients = coefficients.df
+    output$estimates = c(coefficients.df$`L-RESI`, coefficients.df[,"CS-RESI"])
+    names.est = rep(rownames(coefficients.df), 2)
+    names(output$estimates) = names.est
+  }
+
+  # anova
+  if (anova) {
+    # longitudinal RESI
+    anova.tab <- anova(model.full)
+    anova.tab[,'L-RESI'] = chisq2S(anova.tab[,'X2'], anova.tab[,'Df'], N)
+
+    # CS RESI
+    # M: use anova() on new mod with substituted variance
+    anova.tabcs <- suppressMessages(car::Anova(mod_indg, vcov. = cov_ind,
+                                               test.statistic = "Wald"))
+    anova.tab[,'CS-RESI'] = chisq2S(anova.tabcs[,'Chisq'], anova.tabcs[,'Df'], N)
+
+    output$anova = anova.tab
+    output$estimates = c(output$estimates, anova.tab$`L-RESI`,
+                         anova.tab[,"CS-RESI"])
+    names.est = c(names.est, rep(rownames(anova.tab), 2))
+    names(output$estimates) = names.est
+    class(output$anova) = c("anova_resi", class(output$anova))
+  }
+
   output$naive.var = FALSE
   return(output)
 }
 
 #' @describeIn resi_pe RESI point estimation for gee object
 #' @export
-resi_pe.gee <- function(model.full, ...){
-  x = as.matrix(summary(model.full)$coefficients)
-  #sample size
+resi_pe.gee <- function(model.full, data, unbiased = TRUE, ...){
+  # sample size
   N = length(unique(model.full$id))
+  # total num of observations
+  tot_obs = nrow(data)
+
+  # independence model
+  data$new_id = 1:nrow(data)
+  # suppressMessages(capture.output(mod_ind <- update(model.full, id = new_id, data = data),
+  #                                           file =  nullfile()))
+  # # adjust covariance
+  # mod_ind$robust.variance = mod_ind$robust.variance * tot_obs/N
+  # to match geeglm
+  mod_indg = glm(formula = formula(model.full), family = model.full$family, data = data,
+                 contrasts = model.full$contrasts)
+
+  # the var-cov matrix estimate from the independence model
+  # Note: this is the estimate for Cov[(\hat{\beta}_ind - \beta_0)]
+  cov_ind = sandwich::vcovHC(mod_indg, type = "HC0")
+  cov_ind = cov_ind * tot_obs/N
+  # make copy of full model and substitute independence variance
+  mod_ind = model.full
+  mod_ind$robust.variance = cov_ind
+
   output <- list(model.full = list(call = model.full$call, formula = formula(model.full)),
-                 coefficients =  as.data.frame(cbind(x, RESI = RESI::chisq2S(x[, 'Robust z']^2, 1, N))))
+                 estimates = c())
+  names.est = c()
+
+
+
+  # longitudinal RESI
+  # coefficients (z statistics)
+  coefficients.tab <- summary(model.full)$coefficients
+  coefficients.df = data.frame(coefficients.tab[,'Estimate'],
+                               coefficients.tab[,'Robust S.E.'],
+                               coefficients.tab[,'Robust z'],
+                               row.names = rownames(coefficients.tab))
+  colnames(coefficients.df) = c('Estimate', 'Std. Error', 'z value')
+  if (unbiased){
+    coefficients.df[,'L-RESI'] = z2S(coefficients.df[,'z value'], N)
+  } else{
+    coefficients.df[,'L-RESI'] = suppressWarnings(z2S_alt(coefficients.df[,'z value'],
+                                                        N))
+  }
+
+  # CS RESI
+  # M: use new mod with substituted variance
+  coefficients.tabcs = summary(mod_ind)$coefficients
+  z_cs = coefficients.tabcs[,'Robust z']
+  if (unbiased){
+    coefficients.df[,'CS-RESI'] = z2S(z_cs, N)
+  } else{
+    coefficients.df[,'CS-RESI'] = suppressWarnings(z2S_alt(z_cs,
+                                                           N))
+  }
+  output$coefficients = coefficients.df
+  output$estimates = c(coefficients.df$`L-RESI`, coefficients.df[,"CS-RESI"])
+  names.est = rep(rownames(coefficients.df),2)
+  names(output$estimates) = names.est
+
   output$naive.var = FALSE
   return(output)
 }
@@ -586,7 +714,8 @@ resi_pe.gee <- function(model.full, ...){
 #' @describeIn resi_pe RESI point estimation for lme object
 #' @importFrom clubSandwich vcovCR
 #' @export
-resi_pe.lme <- function(model.full, vcovfunc = clubSandwich::vcovCR, vcov.args = list(), ...){
+resi_pe.lme <- function(model.full, anova = TRUE, vcovfunc = clubSandwich::vcovCR,
+                        Anova.args = list(), vcov.args = list(), ...){
   x = as.matrix(summary(model.full)$tTable)
   #sample size
   N = summary(model.full)$dims$ngrps[1]
@@ -604,9 +733,24 @@ resi_pe.lme <- function(model.full, vcovfunc = clubSandwich::vcovCR, vcov.args =
 
   robust.var = diag(vcovfunc2(model.full))
   robust.se = sqrt(robust.var)
+  coefficients.df = as.data.frame(cbind(x, 'Robust.SE' = robust.se,
+                                        'Robust Wald' = (x[, 'Value']^2/robust.var),
+                                        RESI = RESI::chisq2S(x[, 'Value']^2/robust.var, 1, N)))
   output = list(model.full = list(call = model.full$call, formula = formula(model.full)),
-       coefficients =  as.data.frame(cbind(x, 'Robust.SE' = robust.se,
-       'Robust Wald' = (x[, 'Value']^2/robust.var), RESI = RESI::chisq2S(x[, 'Value']^2/robust.var, 1, N))))
+       estimates = coefficients.df$RESI, coefficients = coefficients.df)
+  names(output$estimates) = rownames(coefficients.df)
+
+  # Anova table (Chi sq statistics)
+  if (anova){
+    suppressMessages(anova.tab <- do.call(car::Anova, c(list(mod = model.full, vcov. = vcovfunc2), Anova.args)))
+    anova.tab[,'RESI'] = chisq2S(anova.tab[,'Chisq'], anova.tab[,'Df'], N)
+    names.est = names(output$estimates)
+    output$estimates = c(output$estimates, anova.tab$RESI)
+    names.est = c(names.est, rownames(anova.tab))
+    names(output$estimates) = names.est
+    output$anova = anova.tab
+    class(output$anova) = c("anova_resi", class(output$anova))
+  }
   if(identical(vcovfunc, stats::vcov)){
     output$naive.var = TRUE
   }
@@ -618,14 +762,17 @@ resi_pe.lme <- function(model.full, vcovfunc = clubSandwich::vcovCR, vcov.args =
 
 #' @describeIn resi_pe RESI point estimation for lmerMod object
 #' @export
-resi_pe.lmerMod <- function(model.full, vcovfunc = clubSandwich::vcovCR, vcov.args = list(), ...){
+resi_pe.lmerMod <- function(model.full, anova = TRUE, vcovfunc = clubSandwich::vcovCR,
+                            Anova.args = list(), vcov.args = list(), ...){
   x = as.matrix(summary(model.full)$coefficients)
   #sample size
   N = summary(model.full)$ngrps
 
   # robust se
   if (identical(vcovfunc, stats::vcov)) {
-    output = list(coefficients = cbind(x, 'Wald' = x[, 't value']^2, RESI = RESI::chisq2S(x[, 't value']^2, 1, N)), naive.var = TRUE)
+    coefficients.tab = cbind(x, 'Wald' = x[, 't value']^2, RESI = RESI::chisq2S(x[, 't value']^2, 1, N))
+    output = list(estimates = coefficients.tab[,"RESI"], coefficients = coefficients.tab, naive.var = TRUE)
+    vcovfunc2 = vcovfunc
     } else {
       if (length(vcov.args) == 0){
         if (identical(vcovfunc, clubSandwich::vcovCR)){
@@ -641,8 +788,26 @@ resi_pe.lmerMod <- function(model.full, vcovfunc = clubSandwich::vcovCR, vcov.ar
         do.call(vcovfunc, vcov.args)}
       robust_var = diag(vcovfunc2(model.full))
       robust_se = sqrt(robust_var)
-      output = list(coefficients = cbind(x, 'Robust.SE' = robust_se, 'Robust Wald' = (x[, 'Estimate']^2/robust_var), RESI = RESI::chisq2S(x[, 'Estimate']^2/robust_var, 1, N)), naive.var = FALSE)
-      }
-
+      coefficients.tab = cbind(x, 'Robust.SE' = robust_se,
+                               'Robust Wald' = (x[, 'Estimate']^2/robust_var),
+                               RESI = RESI::chisq2S(x[, 'Estimate']^2/robust_var, 1, N))
+      # model full is not the same process as others, so print method is missing the call
+      output = list(estimates = coefficients.tab[,"RESI"], coefficients = coefficients.tab,
+                    naive.var = FALSE)
+    }
+  names(output$estimates) = rownames(coefficients.tab)
+  # Anova table (Chi sq statistics)
+  if (anova){
+    suppressMessages(anova.tab <- do.call(car::Anova,
+                                          c(list(mod = model.full, vcov. = vcovfunc2), Anova.args)))
+    anova.tab[,'RESI'] = chisq2S(anova.tab[,'Chisq'], anova.tab[,'Df'], N)
+    names.est = names(output$estimates)
+    output$estimates = c(output$estimates, anova.tab$RESI)
+    names.est = c(names.est, rownames(anova.tab))
+    names(output$estimates) = names.est
+    output$anova = anova.tab
+    class(output$anova) = c("anova_resi", class(output$anova))
+  }
+  output$model.full = list(call = model.full@call, formula = formula(model.full))
   return(output)
 }
